@@ -45,7 +45,7 @@
 #include	<time.h>
 #include	<numlib/numlib.h>
 
-static const char rcsid[]="$Id: maildirquota.c,v 1.4 2000/09/04 17:10:28 mrsam Exp $";
+static const char rcsid[]="$Id: maildirquota.c,v 1.5 2000/12/25 19:43:17 mrsam Exp $";
 
 /* Read the maildirsize file */
 
@@ -229,8 +229,8 @@ struct dirent *de;
 	if (stat(checkfolder, &stat_buf) == 0)	/* Go to parent */
 	{
 		strcat(strcpy(checkfolder, dir), "/..");
-		n=maildir_checkquota(checkfolder, maildirsize_fdptr,
-			quota_type, xtra_size, xtra_cnt);
+		n=docheckquota(checkfolder, maildirsize_fdptr,
+			quota_type, xtra_size, xtra_cnt, percentage);
 		free(checkfolder);
 		return (n);
 	}
