@@ -1,4 +1,4 @@
-/* $Id: acconfig.h,v 1.7 2000/04/28 17:17:45 mrsam Exp $ */
+/* $Id: acconfig.h,v 1.8 2001/02/02 13:34:38 mrsam Exp $ */
 
 /* Our package */
 #undef PACKAGE
@@ -64,6 +64,12 @@
 /* Whether maildir quota feature is enabled */
 #undef	MAILDIRQUOTA
 
+/* Whether virtual user database via LDAP feature is enabled */
+#undef USELDAP
+
+/* Where to find the LDAP config file */
+#undef MAILDROPLDAPCONFIG
+
 /* Whether virtual user database feature is enabled */
 #undef	USERDB
 
