@@ -15,6 +15,11 @@
 #if	HAVE_STRINGS_H
 #include	<strings.h>
 #endif
+
+#if	HAVE_LOCALE_H
+#include	<locale.h>
+#endif
+
 #include	<stdlib.h>
 #include	<ctype.h>
 #include	<pwd.h>
@@ -37,7 +42,7 @@ int gethostname(const char *, size_t);
 #endif
 
 
-static const char rcsid[]="$Id: reformime.c,v 1.30 2000/09/14 02:51:40 mrsam Exp $";
+static const char rcsid[]="$Id: reformime.c,v 1.36 2001/04/23 04:05:05 mrsam Exp $";
 
 void rfc2045_error(const char *errmsg)
 {
@@ -64,7 +69,7 @@ FILE	*fp=(FILE *)ptr;
 	while (n)
 	{
 		--n;
-		if (putc(*p++, fp) == -1)
+		if (putc((int)(unsigned char)*p++, fp) == EOF)
 		{
 			perror("write");
 			exit(1);
@@ -352,6 +357,15 @@ char	*dyn_disp_name=0;
 		perror("rfc2047_decode_simple");
 		exit(1);
 	}
+
+	if (strlen(filename_buf) > 32)
+	{
+		p=filename_buf;
+		q=filename_buf + strlen(filename_buf)-32;
+		while ( (*p++ = *q++) != 0)
+			;
+	}
+
 	disposition_filename_s=filename_buf;
 
 	if (ignore_filename)
@@ -865,8 +879,14 @@ char	*mimesection=0;
 int	doinfo=0, dodecode=0, dorewrite=0, dodsn=0, domimedigest=0;
 struct	rfc2045 *p;
 int	rwmode=0;
+int	dovalidate=0;
 void	(*do_extract)(struct rfc2045 *, const char *, int, char **)=0;
 const char *extract_filename=0;
+int rc=0;
+
+#if HAVE_SETLOCALE
+	setlocale(LC_ALL, "C");
+#endif
 
 	for (argn=1; argn<argc; )
 	{
@@ -921,6 +941,9 @@ const char *extract_filename=0;
 		case 'v':
 			printf("%s\n", rcsid);
 			exit(0);
+		case 'V':
+			dovalidate=1;
+			break;
 		default:
 			usage();
 		}
@@ -935,7 +958,12 @@ const char *extract_filename=0;
 	p=read_message();
 
 	if (doinfo)
+	{
 		print_info(p, mimesection);
+		if (do_extract)
+			extract_section(p, mimesection, extract_filename,
+					argc-argn, argv+argn, do_extract);
+	}
 	else if (dodecode)
 		print_decode(p, mimesection);
 	else if (dorewrite)
@@ -945,9 +973,24 @@ const char *extract_filename=0;
 	else if (do_extract)
 		extract_section(p, mimesection, extract_filename,
 			argc-argn, argv+argn, do_extract);
+	else if (dovalidate)
+	{
+		rc=1;
+
+		if (p->rfcviolation & RFC2045_ERR8BITHEADER)
+			printf("ERROR: Illegal 8-bit header.\n");
+		else if (p->rfcviolation & RFC2045_ERR8BITCONTENT)
+			printf("ERROR: Illegal 8-bit content.\n");
+		else if (p->rfcviolation & RFC2045_ERR2COMPLEX)
+			printf("ERROR: MIME complexity.\n");
+		else if (p->rfcviolation & RFC2045_ERRBADBOUNDARY)
+			printf("ERROR: Ambiguous  MIME boundary delimiters.\n");
+		else rc=0;
+
+	}
 	else
 		print_structure(p);
 	rfc2045_free(p);
-	exit(0);
-	return (0);
+	exit(rc);
+	return (rc);
 }
