@@ -1,14 +1,10 @@
-/* DSTART                                                                    */
-/*                                                                           */
-/*           maildrop - mail delivery agent with filtering abilities         */
-/*                                                                           */
-/*  Copyright 1998-1999, Double Precision Inc.                               */
-/*                                                                           */
-/*  This program is distributed under the terms of the GNU General Public    */
-/*  License. See COPYING for additional information.                         */
-/* DEND                                                                      */
 /*
-** $Id: rfc2045.c,v 1.12 1999/11/30 02:35:30 mrsam Exp $
+** Copyright 1998 - 2000 Double Precision, Inc.  See COPYING for
+** distribution information.
+*/
+
+/*
+** $Id: rfc2045.c,v 1.26 2001/04/07 23:31:01 mrsam Exp $
 */
 #if    HAVE_CONFIG_H
 #include       "config.h"
@@ -26,10 +22,10 @@
 
 static char	*rfc2045_defcharset=0;
 
-void rfc2045_enomem()
-{
-	rfc2045_error("Out of memory.");
-}
+extern void rfc2045_enomem();
+
+#define	MAXLEVELS	20
+#define	MAXPARTS	300
 
 /*
 	New RFC2045 structure.
@@ -39,7 +35,11 @@ struct rfc2045 *rfc2045_alloc()
 {
 struct rfc2045 *p=(struct rfc2045 *)malloc(sizeof(struct rfc2045));
 
-	if (!p)	rfc2045_enomem();
+	if (!p)
+	{
+		rfc2045_enomem();
+		return (0);
+	}
 
 	/* Initialize everything to nulls, except for one thing */
 
@@ -87,13 +87,19 @@ char	*v;
 	}
 
 	v=strdup(val);
-	if (!v)	rfc2045_enomem();
+	if (!v)
+	{
+		rfc2045_enomem();
+		return;
+	}
+
 	if (!*p)
 	{
 		if (((*p)=(struct rfc2045attr *)malloc(sizeof(**p))) == 0)
 		{
 			free(v);
 			rfc2045_enomem();
+			return;
 		}
 		memset( (*p), 0, sizeof(**p));
 		if ( ((*p)->name=strdup(name)) == 0)
@@ -102,15 +108,18 @@ char	*v;
 			*p=0;
 			free(v);
 			rfc2045_enomem();
+			return;
 		}
 	}
 	if ( (*p)->value )	free ( (*p)->value );
 	(*p)->value=v;
 }
 
-static const char cb_name[]="boundary";
+/* static const char cb_name[]="boundary"; */
 
-#define	ContentBoundary(p)	(rfc2045_getattr( (p)->content_type_attr, cb_name))
+/* #define	ContentBoundary(p)	(rfc2045_getattr( (p)->content_type_attr, cb_name)) */
+
+#define	ContentBoundary(p)	( (p)->boundary )
 
 /*
 	Unallocate the RFC2045 structure.  Recursively unallocate
@@ -143,7 +152,14 @@ struct rfc2045 *q, *r;
 	rfc2045_freeattr(p->content_type_attr);
 	rfc2045_freeattr(p->content_disposition_attr);
 
+	if (p->content_md5)	free(p->content_md5);
+	if (p->content_base)	free(p->content_base);
+	if (p->content_location)	free(p->content_location);
+	if (p->content_language)	free(p->content_language);
+	if (p->content_id)	free(p->content_id);
+	if (p->content_description)	free(p->content_description);
 	if (p->content_transfer_encoding) free(p->content_transfer_encoding);
+	if (p->boundary) free(p->boundary);
 	if (p->content_type)	free(p->content_type);
 	if (p->mime_version)	free(p->mime_version);
 	if (p->workbuf)		free(p->workbuf);
@@ -169,7 +185,11 @@ void rfc2045_add_buf(
 	char	*p= *bufptr ? (char *)realloc(*bufptr, newsize):
 				(char *)malloc(newsize);
 
-		if (!p)	rfc2045_enomem();
+		if (!p)
+		{
+			rfc2045_enomem();
+			return;
+		}
 		*bufptr=p;
 		*bufsize=newsize;
 	}
@@ -209,7 +229,10 @@ static void set_string(char **p,
 	if (!q)	return;
 
 	if ((*p=(char *)malloc(strlen(q)+1)) == 0)
+	{
 		rfc2045_enomem();
+		return;
+	}
 
 	strcpy(*p, q);
 }
@@ -262,7 +285,7 @@ size_t	l;
 	}
 
 	/*
-	** Are buffer's getting pretty big.  Let's see if we can
+	** Our buffer's getting pretty big.  Let's see if we can
 	** partially handle it.
 	*/
 
@@ -329,6 +352,11 @@ struct rfc2045 *newp;
 
 	/* Initialize source pointers */
 	newp->startpos=newp->endpos=newp->startbody=newp->endbody=startpos;
+
+	while (p->parent)
+		p=p->parent;
+	++p->numparts;
+
 	return (newp);
 }
 
@@ -364,6 +392,19 @@ char *c=p->workbuf;
 size_t	n=cnt-1;	/* Strip \n (we always get at least a \n here) */
 struct rfc2045 *newp;
 struct rfc2045ac *rwp=p->rfc2045acptr;
+unsigned num_levels=0;
+
+size_t	k;
+int	bit8=0;
+
+	if (p->numparts > MAXPARTS)
+	{
+		p->rfcviolation |= RFC2045_ERR2COMPLEX;
+		return;
+	}
+
+	for (k=0; k<cnt; k++)
+		if (c[k] & 0x80)	bit8=1;
 
 	if (n && c[n-1] == '\r')	/* Strip trailing \r */
 		--n;
@@ -372,7 +413,8 @@ struct rfc2045ac *rwp=p->rfc2045acptr;
 	** in a middle of a form-data section.  */
 
 	for (newp=p; newp->lastpart &&
-			!newp->lastpart->workclosed; newp=newp->lastpart)
+			!newp->lastpart->workclosed; newp=newp->lastpart,
+			++num_levels)
 	{
 		if (ContentBoundary(newp) == 0 || newp->workinheader)
 			continue;
@@ -411,6 +453,7 @@ struct rfc2045ac *rwp=p->rfc2045acptr;
 		if (cb == 0 || p->workinheader)
 		{
 			p=p->lastpart;
+			++num_levels;
 			continue;
 		}
 
@@ -447,6 +490,7 @@ struct rfc2045ac *rwp=p->rfc2045acptr;
 			return;
 		}
 		p=p->lastpart;
+		++num_levels;
 	}
 
 	/* Ok, we've found the RFC2045 section that we're working with.
@@ -456,11 +500,21 @@ struct rfc2045ac *rwp=p->rfc2045acptr;
 	if (! p->workinheader)
 	{
 		/* Processing body, just update the counts. */
+
 	size_t cnt_update=cnt;
 
+		if (bit8 && !p->content_8bit &&
+			(p->rfcviolation & RFC2045_ERR8BITCONTENT) == 0)
+		{
+		struct rfc2045 *q;
+
+			for (q=p; q; q=q->parent)
+				q->rfcviolation |= RFC2045_ERR8BITCONTENT;
+		}
+
 		/*
-		** In multiparts, the final newline in a part belongs to the boundary,
-		** otherwise, include it in the text.
+		** In multiparts, the final newline in a part belongs to the
+		** boundary, otherwise, include it in the text.
 		*/
 		if (p->parent && p->parent->content_type &&
 				strncasecmp(p->parent->content_type,
@@ -478,6 +532,14 @@ struct rfc2045ac *rwp=p->rfc2045acptr;
 		return;
 	}
 
+	if (bit8 && (p->rfcviolation & RFC2045_ERR8BITHEADER) == 0)
+	{
+	struct rfc2045 *q;
+
+		for (q=p; q; q=q->parent)
+			q->rfcviolation |= RFC2045_ERR8BITHEADER;
+	}
+
 	/* In the header */
 
 	if ( n == 0 )	/* End of header, body begins.  Parse header. */
@@ -504,6 +566,11 @@ struct rfc2045ac *rwp=p->rfc2045acptr;
 			set_string(&p->content_disposition, 0);
 			rfc2045_freeattr(p->content_disposition_attr);
 			p->content_disposition_attr=0;
+			if (p->boundary)
+			{
+				free(p->boundary);
+				p->boundary=0;
+			}
 		}
 
 		/* Normally, if we don't have a content_type, default it
@@ -528,7 +595,11 @@ struct rfc2045ac *rwp=p->rfc2045acptr;
 
 		if (!p->content_type ||
 			strncmp(p->content_type, "multipart/", 10))
-			rfc2045_setattr(&p->content_type_attr, "boundary", 0);
+		{
+			if (p->boundary)
+				free(p->boundary);
+			p->boundary=0;
+		}
 
 		/* If this section's a message, we will expect to see
 		** more RFC2045 stuff, so create a nested RFC2045 structure,
@@ -540,6 +611,7 @@ struct rfc2045ac *rwp=p->rfc2045acptr;
 		{
 			newp=append_part_noinherit(p, p->startbody);
 			newp->workinheader=1;
+			return;
 		}
 
 		/*
@@ -548,14 +620,16 @@ struct rfc2045ac *rwp=p->rfc2045acptr;
 		** that precedes the first boundary line.
 		*/
 
-		else if (ContentBoundary(p))
+		if (ContentBoundary(p))
 		{
 			newp=append_part(p, p->startbody);
 			newp->workinheader=0;
 			newp->isdummy=1;
-					/* It's easier just to create it. */
+				/* It's easier just to create it. */
+			return;
 		}
-		else if (rwp)
+
+		if (rwp)
 			(*rwp->start_section)(p);
 		return;
 	}
@@ -620,12 +694,8 @@ char	*p;
 	p=( char *)malloc(l);
 	if (!p)
 	{
-		/* The RFC822 structure we've allocated was temporary.
-		** Error condition - free it, so we don't leak memory.
-		*/
-
-		rfc822t_free(h);
 		rfc2045_enomem();
+		return (0);
 	}
 	l=0;
 
@@ -656,7 +726,7 @@ static char *lower_paste_tokens(struct rfc822t *h, int start, int cnt)
 char	*p=paste_tokens(h, start, cnt);
 char	*q;
 
-	for (q=p; *q; q++)
+	for (q=p; q && *q; q++)
 		*q=tolower(*q);
 	return (p);
 }
@@ -672,7 +742,7 @@ static char *lower_paste_token(struct rfc822t *h, int i)
 char *p=paste_token(h, i);
 char *q;
 
-	for (q=p; *q; q++)
+	for (q=p; q && *q; q++)
 		*q=tolower(*q);
 	return (p);
 }
@@ -685,6 +755,12 @@ static void mime_version(struct rfc2045 *, struct rfc822t *);
 static void content_type(struct rfc2045 *, struct rfc822t *);
 static void content_transfer_encoding(struct rfc2045 *, struct rfc822t *);
 static void content_disposition(struct rfc2045 *, struct rfc822t *);
+static void content_id(struct rfc2045 *, struct rfc822t *);
+static void content_description(struct rfc2045 *, const char *);
+static void content_language(struct rfc2045 *, const char *);
+static void content_md5(struct rfc2045 *, const char *);
+static void content_base(struct rfc2045 *, struct rfc822t *);
+static void content_location(struct rfc2045 *, struct rfc822t *);
 
 static void do_header(struct rfc2045 *p)
 {
@@ -697,7 +773,7 @@ char	*t;
 
 	/* Parse the header line according to RFC822 */
 
-	header=rfc822t_alloc(p->header, NULL);
+	header=rfc822t_alloc_new(p->header, NULL, NULL);
 
 	if (!header)	return;	/* Broken header */
 
@@ -710,7 +786,10 @@ char	*t;
 	}
 
 	t=lower_paste_token(header, 0);
-	if (strcmp(t, "mime-version") == 0)
+
+	if (t == 0)
+		;
+	else if (strcmp(t, "mime-version") == 0)
 	{
 		free(t);
 		mime_version(p, header);
@@ -727,6 +806,42 @@ char	*t;
 	{
 		free(t);
 		content_disposition(p, header);
+	} else if (strcmp(t, "content-id") == 0)
+	{
+		free(t);
+		content_id(p, header);
+	} else if (strcmp(t, "content-description") == 0)
+	{
+		free(t);
+		t=strchr(p->header, ':');
+		if (t)	++t;
+		while (t && isspace((int)(unsigned char)*t))
+			++t;
+		content_description(p, t);
+	} else if (strcmp(t, "content-language") == 0)
+	{
+		free(t);
+		t=strchr(p->header, ':');
+		if (t)	++t;
+		while (t && isspace((int)(unsigned char)*t))
+			++t;
+		content_language(p, t);
+	} else if (strcmp(t, "content-base") == 0)
+	{
+		free(t);
+		content_base(p, header);
+	} else if (strcmp(t, "content-location") == 0)
+	{
+		free(t);
+		content_location(p, header);
+	} else if (strcmp(t, "content-md5") == 0)
+	{
+		free(t);
+		t=strchr(p->header, ':');
+		if (t)	++t;
+		while (t && isspace((int)(unsigned char)*t))
+			++t;
+		content_md5(p, t);
 	}
 	else	free(t);
 	rfc822t_free(header);
@@ -738,6 +853,8 @@ static void mime_version(struct rfc2045 *p, struct rfc822t *header)
 {
 char	*vers=paste_tokens(header, 2, header->ntokens-2);
 
+	if (!vers)	return;
+
 	if (p->mime_version)	free(p->mime_version);
 	p->mime_version=vers;
 }
@@ -748,9 +865,14 @@ static void content_transfer_encoding(struct rfc2045 *r,
 char	*p;
 
 	p=lower_paste_tokens(header, 2, header->ntokens-2);
+	if (!p)	return;
+
 	if (r->content_transfer_encoding)
 		free(r->content_transfer_encoding);
 	r->content_transfer_encoding=p;
+
+	if (strcmp(p, "8bit") == 0)
+		r->content_8bit=1;
 }
 
 /* Dig into the content_type header */
@@ -773,6 +895,8 @@ char	*p;
 	/* Everything up to the 1st ; is the content type */
 
 	p=lower_paste_tokens(header, 2, start-2);
+	if (!p)	return;
+
 	(*init_token)(r, p);
 	if (start < header->ntokens) start++;
 
@@ -798,6 +922,7 @@ char	*p;
 			{
 				++j;
 				p=lower_paste_token(header, start);
+				if (!p)	return;
 				(*init_parameter)(r, p, header, j, i-j);
 				free(p);
 			}
@@ -831,11 +956,51 @@ static void save_content_type_parameter(
 {
 char	*p;
 
-	p=strcmp(name, "charset") == 0 || strcmp(name, "boundary") == 0 ?
+	p=strcmp(name, "charset") == 0 ?
 			lower_paste_tokens(header, start, len):
 			paste_tokens(header, start, len);
+	if (!p)	return;
+
 	rfc2045_setattr(&r->content_type_attr, name, p);
 	free(p);
+
+	if (strcmp(name, "boundary") == 0)
+	{
+		struct rfc2045 *q;
+
+		if (r->boundary)
+			free(r->boundary);
+		p=lower_paste_tokens(header, start, len);
+		r->boundary=p;
+
+		/*
+		** Check all the outer MIME boundaries.  If this is a
+		** substring of an outer MIME boundary, or the outer
+		** boundary is a substring of the inner boundary, we
+		** have an ambiguity - see "IMPLEMENTOR'S NOTE" in
+		** section 5.1.1 of RFC 2046.
+		*/
+
+		for (q=r->parent; q; q=q->parent)
+		{
+			const char *a, *b;
+
+			if (!q->boundary)
+				continue;
+
+			for (a=q->boundary, b=p; *a && *b; a++, b++)
+				if (*a != *b)
+					break;
+
+			if (!*a || !*b)
+			{
+				while (q->parent)
+					q=q->parent;
+				q->rfcviolation |= RFC2045_ERRBADBOUNDARY;
+				break;
+			}
+		}
+	}
 }
 
 /* Dig into content-disposition */
@@ -864,10 +1029,127 @@ static void save_content_disposition_parameter(
 char	*p;
 
 	p=paste_tokens(header, start, len);
+	if (!p)	return;
+
 	rfc2045_setattr(&r->content_disposition_attr, name, p);
 	free(p);
 }
 
+char *rfc2045_related_start(const struct rfc2045 *p)
+{
+const char *cb=rfc2045_getattr( p->content_type_attr, "start");
+struct	rfc822t *t;
+struct	rfc822a	*a;
+int	i;
+
+	if (!cb || !*cb)	return (0);
+
+	t=rfc822t_alloc_new(cb, 0, NULL);
+	if (!t)
+	{
+		rfc2045_enomem();
+		return(0);
+	}
+
+	a=rfc822a_alloc(t);
+	if (!a)
+	{
+		rfc822t_free(t);
+		rfc2045_enomem();
+		return (0);
+	}
+	for (i=0; i<a->naddrs; i++)
+		if (a->addrs[i].tokens)
+		{
+		char	*s=rfc822_getaddr(a, i);
+
+			rfc822a_free(a);
+			rfc822t_free(t);
+			if (!s)
+				rfc2045_enomem();
+			return (s);
+		}
+
+	rfc822a_free(a);
+	rfc822t_free(t);
+	return (0);
+}
+
+static void content_id(struct rfc2045 *p, struct rfc822t *t)
+{
+struct	rfc822a	*a=rfc822a_alloc(t);
+int	i;
+
+	if (!a)
+	{
+		rfc2045_enomem();
+		return;
+	}
+
+	for (i=0; i<a->naddrs; i++)
+		if (a->addrs[i].tokens)
+		{
+		char	*s=rfc822_getaddr(a, i);
+
+			if (!s)
+			{
+				rfc822a_free(a);
+				rfc2045_enomem();
+				return;
+			}
+			if (p->content_id)
+				free(p->content_id);
+			p->content_id=s;
+			break;
+		}
+
+	rfc822a_free(a);
+}
+
+static void content_description(struct rfc2045 *p, const char *s)
+{
+	if (s && *s)
+		set_string(&p->content_description, s);
+}
+
+static void content_language(struct rfc2045 *p, const char *s)
+{
+	if (s && *s)
+		set_string(&p->content_language, s);
+}
+
+static void content_md5(struct rfc2045 *p, const char *s)
+{
+	if (s && *s)
+		set_string(&p->content_md5, s);
+}
+
+static void content_base(struct rfc2045 *p, struct rfc822t *t)
+{
+char	*s;
+int	i;
+
+	for (i=0; i<t->ntokens; i++)
+		if (t->tokens[i].token == '"')
+			t->tokens[i].token=0;
+
+	s=paste_tokens(t, 2, t->ntokens-2);
+	set_string(&p->content_base, s);
+}
+
+static void content_location(struct rfc2045 *p, struct rfc822t *t)
+{
+char	*s;
+int	i;
+
+	for (i=0; i<t->ntokens; i++)
+		if (t->tokens[i].token == '"')
+			t->tokens[i].token=0;
+
+	s=paste_tokens(t, 2, t->ntokens-2);
+	set_string(&p->content_location, s);
+}
+
 /* -------------------- */
 
 #define	GETINFO(s, def) ( (s) && (*s) ? (s):def)
@@ -901,19 +1183,31 @@ void rfc2045_setdefaultcharset(const char *charset)
 {
 char	*p=strdup(charset);
 
-	if (!p)	rfc2045_enomem();
+	if (!p)
+	{
+		rfc2045_enomem();
+		return;
+	}
+
 	if (rfc2045_defcharset)	free(rfc2045_defcharset);
 	rfc2045_defcharset=p;
 }
 
 const char *rfc2045_boundary(const struct rfc2045 *p)
 {
-const char *cb=ContentBoundary(p);
+const char *cb=rfc2045_getattr( p->content_type_attr, "boundary");
 
 	if (!cb)	cb="";
 	return (cb);
 }
 
+int rfc2045_isflowed(const struct rfc2045 *p)
+{
+	const char *cb=rfc2045_getattr(p->content_type_attr, "format");
+
+	return (cb && strcmp(cb, "flowed") == 0);
+}
+
 void rfc2045_dispositioninfo(const struct rfc2045 *p,
 	const char **disposition_s,
 	const char **disposition_name_s,
@@ -934,6 +1228,26 @@ const	char *q=rfc2045_getattr(p->content_type_attr, "name");
 	return (q);
 }
 
+const char *rfc2045_content_id(const struct rfc2045 *p)
+{
+	return (p->content_id ? p->content_id:"");
+}
+
+const char *rfc2045_content_description(const struct rfc2045 *p)
+{
+	return (p->content_description ? p->content_description:"");
+}
+
+const char *rfc2045_content_language(const struct rfc2045 *p)
+{
+	return (p->content_language ? p->content_language:"");
+}
+
+const char *rfc2045_content_md5(const struct rfc2045 *p)
+{
+	return (p->content_md5 ? p->content_md5:"");
+}
+
 void rfc2045_mimepos(const struct rfc2045 *p,
 	off_t *start_pos, off_t *end_pos, off_t *start_body,
 	off_t *nlines, off_t *nbodylines)
@@ -950,6 +1264,11 @@ void rfc2045_mimepos(const struct rfc2045 *p,
 		if (*nbodylines) --*nbodylines;
 	}
 	*start_body=p->startbody;
+
+	if (*start_body == *start_pos)	/* No header */
+	{
+		*start_body= *end_pos;
+	}
 }
 
 unsigned rfc2045_mimepartcount(const struct rfc2045 *p)
