@@ -1,16 +1,12 @@
-/* DSTART                                                                    */
-/*                                                                           */
-/*           maildrop - mail delivery agent with filtering abilities         */
-/*                                                                           */
-/*  Copyright 1998-1999, Double Precision Inc.                               */
-/*                                                                           */
-/*  This program is distributed under the terms of the GNU General Public    */
-/*  License. See COPYING for additional information.                         */
-/* DEND                                                                      */
 #ifndef	userdb_h
 #define	userdb_h
 
-static const char userdb_rcsid_h[]="$Id: userdb.h,v 1.1 1999/08/16 16:32:16 mrsam Exp $";
+/*
+** Copyright 1998 - 1999 Double Precision, Inc.
+** See COPYING for distribution information.
+*/
+
+static const char userdb_rcsid_h[]="$Id: userdb.h,v 1.3 1999/12/15 05:22:48 mrsam Exp $";
 
 #include	<sys/types.h>
 
@@ -45,6 +41,7 @@ char *userdb_gets(const char *,	/* The record */
 
 struct userdbs {
 	char *udb_name;	   /* Account name, ONLY set by userdb_createsuid */
+	char *udb_gecos;	/* GECOS */
 	char *udb_dir;		/* Home directory */
 	char *udb_shell;	/* Shell */
 	char *udb_mailbox;	/* Default mailbox */
